<?php
declare(strict_types=1);

namespace App\Controller;

use Cake\Datasource\ConnectionManager;

/**
 * Rates Controller
 *
 * @property \App\Model\Table\RatesTable $Rates
 */
class RatesController extends AppController
{
    /**
     * Index method
     *
     * @return \Cake\Http\Response|null|void Renders view
     */
    public function index()
    {

        $connection = ConnectionManager::get('default');
        $from_state = $connection->execute("SELECT distinct from_state FROM rates")->fetchAll('assoc');
        $fromStates = [];
        foreach ($from_state as $k => $state) {
            $fromStates[$state['from_state']] = $state['from_state'];
        }
        $to_state = $connection->execute("SELECT distinct to_state FROM rates")->fetchAll('assoc');
        $toStates = [];
        foreach ($to_state as $k => $state) {
            $toStates[$state['to_state']] = $state['to_state'];
        }
        $this->set(compact('fromStates', 'toStates'));

        if ($this->request->is('post')) {
            $data = $this->request->getData();
            if ( ($data['from_state'] == '') || ($data['to_state'] == '') ) {
                $this->Flash->error(__('Please Select From and To State'));
                return $this->redirect(['action' => 'index']);
            }
            $results = $this->Rates->find()->where(['from_state' => $data['from_state'], 'to_state' => $data['to_state']])->first();
            $this->set(compact('results'));
        }
    }

    /**
     * View method
     *
     * @param string|null $id Rate id.
     * @return \Cake\Http\Response|null|void Renders view
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view()
    {

        $this->set('title', 'View Other Information');
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null|void Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $rate = $this->Rates->newEmptyEntity();
        if ($this->request->is('post')) {
            $rate = $this->Rates->patchEntity($rate, $this->request->getData());
            if ($this->Rates->save($rate)) {
                $this->Flash->success(__('The rate has been saved.'));

                return $this->redirect(['action' => 'index']);
            }
            $this->Flash->error(__('The rate could not be saved. Please, try again.'));
        }
        $this->set(compact('rate'));
    }

    /**
     * Edit method
     *
     * @param string|null $id Rate id.
     * @return \Cake\Http\Response|null|void Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $rate = $this->Rates->get($id, contain: []);
        if ($this->request->is(['patch', 'post', 'put'])) {
            $rate = $this->Rates->patchEntity($rate, $this->request->getData());
            if ($this->Rates->save($rate)) {
                $this->Flash->success(__('The rate has been saved.'));

                return $this->redirect(['action' => 'edit/'.$id]);
            }
            $this->Flash->error(__('The rate could not be saved. Please, try again.'));
        }
        $this->set(compact('rate'));
    }

    /**
     * Delete method
     *
     * @param string|null $id Rate id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
        $this->request->allowMethod(['post', 'delete']);
        $rate = $this->Rates->get($id);
        if ($this->Rates->delete($rate)) {
            $this->Flash->success(__('The rate has been deleted.'));
        } else {
            $this->Flash->error(__('The rate could not be deleted. Please, try again.'));
        }

        return $this->redirect(['action' => 'index']);
    }
}
